export function countItems<T>(arr: T[]) {
  const map = new Map<T, number>();
  for (const item of arr) {
    map.set(item, (map.get(item) || 0) + 1);
  }
  return map;
}

export class BufferWriter {
  buffer: Uint8Array;
  pointer = 0;
  constructor(length: number) {
    this.buffer = new Uint8Array(length);
  }
  writeUint32LE(value: number) {
    this.buffer[this.pointer++] = value & 0xff;
    this.buffer[this.pointer++] = (value >> 8) & 0xff;
    this.buffer[this.pointer++] = (value >> 16) & 0xff;
    this.buffer[this.pointer++] = (value >> 24) & 0xff;
    return this;
  }

  writeUint32BE(value: number) {
    this.buffer[this.pointer++] = (value >> 24) & 0xff;
    this.buffer[this.pointer++] = (value >> 16) & 0xff;
    this.buffer[this.pointer++] = (value >> 8) & 0xff;
    this.buffer[this.pointer++] = value & 0xff;
    return this;
  }

  writeUint16LE(value: number) {
    this.buffer[this.pointer++] = value & 0xff;
    this.buffer[this.pointer++] = (value >> 8) & 0xff;
    return this;
  }

  writeUint16BE(value: number) {
    this.buffer[this.pointer++] = (value >> 8) & 0xff;
    this.buffer[this.pointer++] = value & 0xff;
    return this;
  }

  writeUint8(value: number) {
    this.buffer[this.pointer++] = value & 0xff;
    return this;
  }

  writeString(str: string) {
    for (let i = 0; i < str.length; i++) {
      this.buffer[this.pointer++] = str.charCodeAt(i);
    }
    return this;
  }
}
